package cz.fidentis.analyst.visitors.mesh;

import cz.fidentis.analyst.visitors.mesh.HausdorffDistMeshVisitor;
import cz.fidentis.analyst.mesh.core.CornerTableRow;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshFacetImpl;
import cz.fidentis.analyst.mesh.core.MeshPointImpl;
import java.util.List;
import java.util.Map;
import javax.vecmath.Vector3d;
import static org.junit.jupiter.api.Assertions.assertEquals;
import org.junit.jupiter.api.Test;

public class HausdorffDistMeshVisitorTest {

    private MeshFacet getTrivialFacet(double offset, double size) {
        MeshFacet facet = new MeshFacetImpl();
        facet.addVertex(new MeshPointImpl(new Vector3d(0, 0, offset), new Vector3d(0, 0, 1), new Vector3d()));
        facet.addVertex(new MeshPointImpl(new Vector3d(size, 0, offset), new Vector3d(0, 0, 1), new Vector3d()));
        facet.addVertex(new MeshPointImpl(new Vector3d(0, size, offset), new Vector3d(0, 0, 1), new Vector3d()));

        facet.getCornerTable().addRow(new CornerTableRow(0, -1));
        facet.getCornerTable().addRow(new CornerTableRow(1, -1));
        facet.getCornerTable().addRow(new CornerTableRow(2, -1));

        return facet;
    }

    @Test
    public void visitToVerticesTest() {
        MeshFacet mainFacet = getTrivialFacet(1, 1);
        MeshFacet comparedFacet = getTrivialFacet(1.5, 1);

        HausdorffDistMeshVisitor hausdorffDistance = new HausdorffDistMeshVisitor(mainFacet, false);
        hausdorffDistance.visitMeshFacet(comparedFacet);
        
        Map<MeshFacet, List<Double>> map = hausdorffDistance.getDistances();
        List<Double> results = map.get(mainFacet);
        for (int i = 0; i < mainFacet.getNumberOfVertices(); i++) {
            assertEquals(0.5, results.get(i));
        }
    }

    @Test
    public void visitToVerticesBehindMeshTest() {
        MeshFacet mainFacet = getTrivialFacet(1, 1);
        MeshFacet comparedFacet = getTrivialFacet(-1.5, 1);

        HausdorffDistMeshVisitor hausdorffDistance = new HausdorffDistMeshVisitor(mainFacet, false);
        hausdorffDistance.visitMeshFacet(comparedFacet);

        Map<MeshFacet, List<Double>> map = hausdorffDistance.getDistances();
        List<Double> results = map.get(mainFacet);
        for (int i = 0; i < mainFacet.getNumberOfVertices(); i++) {
            assertEquals(2.5, results.get(i));
        }
    }

    @Test
    public void visitToVerticesRelativeDistanceTest() {
        MeshFacet mainFacet = getTrivialFacet(1, 1);
        MeshFacet comparedFacet = getTrivialFacet(1.5, 1);

        HausdorffDistMeshVisitor hausdorffDistance = new HausdorffDistMeshVisitor(mainFacet, true);
        hausdorffDistance.visitMeshFacet(comparedFacet);

        Map<MeshFacet, List<Double>> map = hausdorffDistance.getDistances();
        List<Double> results = map.get(mainFacet);
        for (int i = 0; i < mainFacet.getNumberOfVertices(); i++) {
            assertEquals(0.5, results.get(i));
        }
    }

    @Test
    public void visitToVerticesBehindMeshRelativeDistanceTest() {
        MeshFacet mainFacet = getTrivialFacet(1, 1);
        MeshFacet comparedFacet = getTrivialFacet(-1.5, 1);

        HausdorffDistMeshVisitor hausdorffDistance = new HausdorffDistMeshVisitor(mainFacet, true);
        hausdorffDistance.visitMeshFacet(comparedFacet);

        Map<MeshFacet, List<Double>> map = hausdorffDistance.getDistances();
        List<Double> results = map.get(mainFacet);
        for (int i = 0; i < mainFacet.getNumberOfVertices(); i++) {
            assertEquals(-2.5, results.get(i));
        }
    }

}
