package cz.fidentis.analyst.visitors.mesh;

import cz.fidentis.analyst.kdtree.KdNode;
import cz.fidentis.analyst.mesh.core.*;
import cz.fidentis.analyst.mesh.io.MeshObjLoader;
import cz.fidentis.analyst.kdtree.KdTree;
import org.junit.jupiter.api.Test;
import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

import static org.junit.jupiter.api.Assertions.*;

public class KdTreeBuildVisitorTest {

    Path testFileDirectory = Paths.get("src", "test", "resources", "cz", "fidentis", "analyst");

    @Test
    void testCreateVisitorsAndGetKdTree() throws IOException {
        MeshModel m = MeshObjLoader.read(new File(testFileDirectory.toFile(), "IcoSphere-20.obj"));
        assertNotNull(m);
        KdTreeBuildVisitor visitor = new KdTreeBuildVisitor();
        m.accept(visitor);
        assertEquals(1, m.getFacets().size());

        List<MeshPoint> points = new LinkedList<>();

        for(MeshFacet facet : m.getFacets()){
            points.addAll(facet.getVertices());
        }

        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        KdTree tree= visitor.getKdTree();
        
        KdNode node = tree.closestNode(p);
        assertNotNull(node);
        assertEquals(node.get3dLocation(), p.getPosition());
    }

    @Test
    void testCreateVisitorsAndGetKdTreeDuplicate() throws IOException {
        MeshModel m = MeshObjLoader.read(new File(testFileDirectory.toFile(), "IcoSphere-20.obj"));
        assertNotNull(m);
        KdTreeBuildVisitor visitor = new KdTreeBuildVisitor();
        m.accept(visitor);
        assertEquals(1, m.getFacets().size());

        List<MeshPoint> points = new LinkedList<>();
        MeshFacet secondFacet = new MeshFacetImpl();
        for(MeshFacet facet : m.getFacets()){
            points.addAll(facet.getVertices());

            for(MeshPoint point : facet.getVertices()) {
                secondFacet.addVertex(point);
            }
        }


        visitor.visitMeshFacet(secondFacet);
        Random r = new Random();

        MeshPoint p = points.get(r.nextInt(points.size()));
        KdTree tree= visitor.getKdTree();

        //System.out.println(tree);

        KdNode node = tree.closestNode(p);
        assertNotNull(node);
        assertEquals(node.get3dLocation(), p.getPosition());
        assertEquals(2, node.getFacets().size());
    }

}