package cz.fidentis.analyst.visitors.mesh;

import cz.fidentis.analyst.mesh.core.CornerTableRow;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshFacetImpl;
import cz.fidentis.analyst.mesh.core.MeshPoint;
import cz.fidentis.analyst.mesh.core.MeshPointImpl;
import java.util.List;
import javax.vecmath.Vector3d;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import org.junit.jupiter.api.Test;

public class Point2MeshVisitorTest {
    private MeshFacet getTrivialFacet(double offset, double size) {
        MeshFacet facet = new MeshFacetImpl();
        facet.addVertex(new MeshPointImpl(new Vector3d(0, 0, offset), new Vector3d(0, 0, 1), new Vector3d()));
        facet.addVertex(new MeshPointImpl(new Vector3d(size, 0, offset), new Vector3d(0, 0, 1), new Vector3d()));
        facet.addVertex(new MeshPointImpl(new Vector3d(0, size, offset), new Vector3d(0, 0, 1), new Vector3d()));

        facet.getCornerTable().addRow(new CornerTableRow(0, -1));
        facet.getCornerTable().addRow(new CornerTableRow(1, -1));
        facet.getCornerTable().addRow(new CornerTableRow(2, -1));

        return facet;
    }

    @Test
    public void absoluteDistTest() {
        MeshFacet facet = getTrivialFacet(1.5, 1);
        MeshPoint point = new MeshPointImpl(new Vector3d(0,0,0), new Vector3d(0,0,-1), new Vector3d());
        
        Point2MeshVisitor vis = new Point2MeshVisitor(point, false);
        facet.accept(vis);
        assertEquals(1.5, vis.getDistance());
        
        List<Integer> closestPoints = vis.getIndices();
        List<MeshFacet> closestMeshes = vis.getClosestFacets();
        assertNotNull(closestPoints);
        assertNotNull(closestMeshes);
        assertEquals(1, closestPoints.size());
        assertEquals(1, closestMeshes.size());
        assertEquals(facet, closestMeshes.get(0));
        assertEquals(0, closestPoints.get(0));
    }

    @Test
    public void relativeDistTest() {
        MeshFacet facet = getTrivialFacet(1.5, 1);
        MeshPoint point = new MeshPointImpl(new Vector3d(0,0,0), new Vector3d(0,0,-1), new Vector3d());
        
        Point2MeshVisitor vis = new Point2MeshVisitor(point, true);
        facet.accept(vis);
        assertEquals(-1.5, vis.getDistance());
        
        List<Integer> closestPoints = vis.getIndices();
        List<MeshFacet> closestMeshes = vis.getClosestFacets();
        assertNotNull(closestPoints);
        assertNotNull(closestMeshes);
        assertEquals(1, closestPoints.size());
        assertEquals(1, closestMeshes.size());
        assertEquals(facet, closestMeshes.get(0));
        assertEquals(0, closestPoints.get(0));
    }
    
    @Test
    public void exactMatchTest() {
        MeshFacet facet = getTrivialFacet(1.5, 1);
        MeshPoint point = new MeshPointImpl(new Vector3d(0, 0, 1.5), new Vector3d(0, 0, 1), new Vector3d());
        
        System.out.println(facet.getVertices());
        
        Point2MeshVisitor vis = new Point2MeshVisitor(point, false);
        facet.accept(vis);
        assertEquals(0, vis.getDistance());
        
        List<Integer> closestPoints = vis.getIndices();
        List<MeshFacet> closestMeshes = vis.getClosestFacets();
        assertNotNull(closestPoints);
        assertNotNull(closestMeshes);
        assertEquals(1, closestPoints.size());
        assertEquals(1, closestMeshes.size());
        assertEquals(facet, closestMeshes.get(0));
        assertEquals(0, closestPoints.get(0));
    }
}
