package cz.fidentis.analyst.mesh;

import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.mesh.core.MeshModel;

/**
 * An object, when instatiated, can be applied to multiple meshes via
 * @see cz.fidentis.analyst.mesh.core.MeshModel#accept() or
 * @see cz.fidentis.analyst.mesh.core.MeshFacet#accept(). It inspects the state
 * of these meshes one by one, and (cumulatevely) computes some results.
 * 
 * Implement this interface whenever you want to define new algorithm over a mesh.
 * 
 * @author Radek Oslejsek
 */
public interface MeshVisitor {
    
    /**
     * Visits a mesh model. Ba default, this method visits 
     * all facets of the model automatically.
     * 
     * @param model Mesh model to be visited.
     */
    default void visitMeshModel(MeshModel model) {
        for (MeshFacet facet: model.getFacets()) {
            facet.accept(this);
        }
    }
   
    /**
     * Visits a mesh facet.
     * 
     * @param facet Mesh facet to be visited.
     */
    void visitMeshFacet(MeshFacet facet);
}
