package cz.fidentis.analyst.mesh.core;

import java.util.List;
import cz.fidentis.analyst.mesh.MeshVisitor;

/**
 * An ancapsulated mesh plate (with shared vertices).
 *
 * @author Matej Lukes
 */
public interface MeshFacet extends Iterable<MeshTriangle> {
    
    /**
     * returns vertex of specified index
     *
     * @param index index of vertex
     * @return vertex
     */
    MeshPoint getVertex(int index);

    /**
     * adds vertex to MeshFacet
     *
     * @param point new vertex
     */
    void addVertex(MeshPoint point);

    /**
     * returns number of vertices in MeshFacet
     *
     * @return number of vertices
     */
    int getNumberOfVertices();

    /**
     * returns list of vertices in MeshFacet
     *
     * @return list if vertices
     */
    List<MeshPoint> getVertices();

    /**
     * returns Corner Table representing MeshFacet
     *
     * @return corner table
     */
    CornerTable getCornerTable();
    
    /**
     * Returns true if normals of vertices are calculated.
     * @return true if normals of vertices are calculated.
     */
    boolean hasVertexNormals();
    
    /**
     * Calculates normals of vertices from normals of triangles.
     */
    void calculateVertexNormals();
    
    /**
     * Returns number of triangles.
     * 
     * @return number of triangles
     */
    int getNumTriangles();
    
    /**
     * Entry point for visitors. 
     * 
     * @param visitor Visitor
     */
    void accept(MeshVisitor visitor);
}
