package cz.fidentis.analyst.mesh.core;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 *
 * @author Natália Bebjaková
 * 
 * Adapter for the corner table representing a single triangle of the @code{MeshFacet}.
 */

public class MeshTriangle implements Iterable<MeshPoint> { 
    
    public final MeshPoint vertex1;
    public final MeshPoint vertex2;
    public final MeshPoint vertex3;
    
    /**
     * Creates new triangle
     * 
     * @param v1 first vertex 
     * @param v2 second vertex
     * @param v3 third vertex
     */        
    public MeshTriangle(MeshPoint v1, MeshPoint v2, MeshPoint v3) {
        this.vertex1 = v1;
        this.vertex2 = v2; 
        this.vertex3 = v3;
    }

    @Override
    public Iterator<MeshPoint> iterator() {
        return new Iterator<MeshPoint>() {
            private int counter = 0;
            
            @Override
            public boolean hasNext() {
                return counter < 3;
            }

            @Override
            public MeshPoint next() {
                if (!hasNext()) {
                    throw new NoSuchElementException();                    
                }
                switch (counter++) {
                    case 0:
                        return vertex1;
                    case 1:
                        return vertex2;
                    case 2:
                        return vertex3;
                    default:
                        return null;
                }                
            }
        };
    }
    
}
